@@ -396,7 +396,7 @@ class PWLCalibration(keras.layers.Layer):
     if inputs.dtype != self._interpolation_keypoints.dtype:
       raise ValueError("dtype(%s) of input to PWLCalibration layer does not "
                        "correspond to dtype(%s) of keypoints. You can enforce "
-                       "dtype of keypoints be explicitly providing 'dtype' "
+                       "dtype of keypoints by explicitly providing 'dtype' "
                        "parameter to layer constructor or by passing keypoints "
                        "in such format which by default will be converted into "
                        "desired one." %
@@ -475,13 +475,13 @@ class PWLCalibration(keras.layers.Layer):
 
     In graph mode builds and returns list of assertion ops. Note that ops will
     be created at the moment when this function is being called.
-    In eager mode directly executes assetions.
+    In eager mode directly executes assertions.
 
     Args:
       eps: Allowed constraints violation.
 
     Returns:
-      List of assertion ops in graph mode or immideately asserts in eager mode.
+      List of assertion ops in graph mode or immediately asserts in eager mode.
     """
     # Assert by computing outputs for keypoints and testing them against
     # constraints.
@@ -569,7 +569,7 @@ class UniformOutputInitializer(keras.initializers.Initializer):
     """Returns weights of PWL calibration layer.
 
     Args:
-      shape: Must be rank-2 tensor with of shape `(k, units)` where `k >= 2`.
+      shape: Must be a collection of the form `(k, units)` where `k >= 2`.
       dtype: Standard Keras initializer param.
       partition_info: Standard Keras initializer param.
 
